#! /bin/sh
# @(#)installmidas.sh	1.1 (ESO-IPG) 07/12/91 14:16:01
# .COPYRIGHT: Copyright (c) 1988 European Southern Observatory,
#                                         all rights reserved
# .TYPE           command
# .NAME           installmidas.sh
# .LANGUAGE       shell script
# .ENVIRONMENT    Unix Systems. Executable under SHELL and C-SHELL
# .COMMENTS       Installation procedure of the MIDAS system starting from
#                 scratch.
#                 
#                 Usage: installmidas [options]
#                 
#                    options: are options for the "make" command. This option
#                             will be exported to all the directories of the
#                             installation procedure.
#			      Multiple definitions for keywords must be:
#				e.g.	CFLAGS=\"-O -g -I/usr/include\"
#			      to avoid quotes to be interpreted by the shell.
#
#                 The command will execute the following steps:
#
#                    - First of all, it checks if the $MIDASHOME and $MIDVERS
#                      shell parameters are defined. If not.. Bye.
#                        
#                    - Compiles, links and executes a Fortran to C test to 
#		       choose the proper interface.
#                        
#                    - $MIDASHOME/local/make_options is read if exists. It
#                      contains local options for "make" commands.a
#                        
#                    - All the directories involved in the installation are
#                      read from: $MID_INSTALL/midas.cnf. To add a new 
#		       directory in the installation edit this file
#                      
#                    - All the directories involved in the installation are
#                      cleaned by running "make options clean" command. 
#
#                    - Run "make options" in all the directories.
#
#                    - Check executables and library files.
#
# .REMARKS        The external variable $MIDASHOME and $MIDVERS must be set 
#                 before running this script. They can be defined in the 
#                 ".profile" or ".cshrc" and executed when you login in
#                 the MIDAS account.
#
# .AUTHOR         Carlos Guirao
# .VERSION 1.1    880588:	Implementation
# .VERSION 2.0    880623:	Adding local files
# .VERSION 2.1    880830:	Adding a "clean" argument to remove
#				files after compilation.
# .VERSION 2.2    881014:	Cosmetic changes
# .VERSION 2.3    891013:	Deleting the "clean" argument.
# .VERSION 2.4    891027:	Now in $MIDAHOME/$MIDVERS/install/unix
# .VERSION 2.5    900422:	Modifying the ftoc tests.
# .VERSION 3.1    901019:	Removing the pre_files step.
# .VERSION 3.2    901112:	Including computer.h installation
# .VERSION 3.3    910219:	Removing SYSTEM name argument
# .VERSION 3.4    910311:	Geting MIDASHOME & MIDVERS 
# .VERSION 4.1    910607:	ftoc test on $MIDASHOME/$MIDVERS/system/ftoc

#
# First of all, goto the configmidas directory adn
# <dirname> & <basename> commands emulated with <sed>
#
#cd `dirname $0`
cd `echo $0 | sed -e 's/[^\/]*$//' -e 's/^$/./' -e 's/\/$//'`
MID_INSTALL=`pwd`
VERSDIR=`echo $MID_INSTALL | sed 's/\/install\/unix$//'`
#MIDVERS=`basename $VERSDIR`
MIDVERS=`echo $VERSDIR | sed -e 's/^.*\///'`
#MIDASHOME=`dirname $VERSDIR`
MIDASHOME=`echo $VERSDIR | sed -e 's/[^\/]*$//' -e 's/^$/./' -e 's/\/$//'`
export MIDASHOME MIDVERS MID_INSTALL

echo "*********************************************************************"
echo "******************** MIDAS INSTALLATION PROCEDURE *******************"
echo "*********************************************************************"
echo "******************** MIDASHOME: $MIDASHOME  VERSION: $MIDVERS"
echo "******************** DATE:  `date`"
echo "******************** HOSTNAME: `hostname`"
echo "******************** COMMAND: $0 $*"
echo ""

#
# Local options for making commands will be read if they exist in the
# $MIDASHOME/local/make_options file.
# Other options can be set as arguments in the command line of this script 
#
# These options will replace in $MIDASHOME/local/make_options file those 
# definitions with the same name:
# Multiple definitions must be between \" e.g. CFLAGS=\"-O -g -I/local/incl\"
#
if [ -r $MIDASHOME/local/make_options ]
then
	echo ""
	echo "*************************************************"
	echo "********* READING YOUR LOCAL OPTIONS ************"
	echo "*************************************************"
	echo "********* DATE:  `date`"
	echo ""
	LOCAL_OPT=`cat $MIDASHOME/local/make_options | awk -F# '{print $1}'`
	echo "$LOCAL_OPT"
	echo ""
fi

echo ""
echo "*************************************************"
echo "********* REMOVING DEPENDENT FILES **************"
echo "*************************************************"
echo "********* DATE:  `date`"
for i in `cat $MID_INSTALL/midas.cnf`
do
	# if line in comments, skip
	comment=`echo $i | sed 's/^\(.\).*/\1/'`
	# echo "comment=$comment"
	if [ $comment = "#" ]
	then
		continue
	fi
	dir=`eval echo $i`
	cd $dir
	echo ""
	echo "**********************************************"
	echo "****** REMOVING FILES IN $dir"
	echo "**********************************************"
	eval make "$@" $LOCAL_OPT clean
	if [ $dir = $MIDASHOME/$MIDVERS/libsrc/ftoc ]
	then 				# removes Fortran to C dependent files
		eval make "$@" $LOCAL_OPT clean_dep
	fi
	if [ $dir = $MIDASHOME/$MIDVERS/libsrc/idi/cidi ]
	then				# removes Fortran to C dependent files
		eval make "$@" $LOCAL_OPT clean_dep
	fi
	if [ $dir = $MIDASHOME/$MIDVERS/libsrc/os/unix ]
	then				# installs version BSD or SYSTEM FIVE
		eval make "$@" $LOCAL_OPT clean_dep
		eval make "$@" $LOCAL_OPT install
	fi
	if [ $dir = $MIDASHOME/$MIDVERS/libsrc/agl ]
	then
		eval make "$@" $LOCAL_OPT clean_dep
	fi
	if [ $dir = $MIDASHOME/$MIDVERS/prim/display/libsrc ]
	then
		eval make "$@" $LOCAL_OPT clean_dep
	fi
done

echo ""
echo "*************************************************"
echo "********** INSTALLING........ *******************"
echo "*************************************************"
echo "********** DATE:  `date`"
for i in `cat $MID_INSTALL/midas.cnf`
do
	# if line in comments, skip
	comment=`echo $i | sed 's/^\(.\).*/\1/'`
	# echo "comment=$comment"
	if [ $comment = "#" ]
	then
		continue
	fi

	dir=`eval echo $i`
	cd $dir
	echo ""
	echo "**********************************************"
	echo "****** INSTALLATION PROCEDURE IN $dir"
	echo "**********************************************"
	echo "****** DATE:  `date`"
	eval make "$@" $LOCAL_OPT all
done

echo ""
echo "*********************************************************************"
echo "******************** END INSTALLATION PROCEDURE *********************"
echo "*********************************************************************"
echo "********** END DATE:  `date`"
echo "********** SYSTEM: $SYSTEM"
