/* @(#)midmessage.c	18.1.1.2 (ESO-DMD) 10/30/02 09:01:53 */
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public
  License along with this program; if not, write to the Free
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge,
  MA 02139, USA.

  Correspondence concerning ESO-MIDAS should be addressed as follows:
        Internet e-mail: midas@eso.org
        Postal address: European Southern Observatory
                        Data Management Division
                        Karl-Schwarzschild-Strasse 2
                        D 85748 Garching bei Muenchen
                        GERMANY
===========================================================================*/

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.LANGUAGE      C
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 .IDENT		MIDAS Monitor module pipe_message()
 .KEYWORDS	Put a formatted string to stdout and MIDAS log
 .INPUT		ID             (int) message identifier
		               ( 1: ERROR, 2: WARNING, 3,4: INFO)
			       negative id has the same meaning except that
			       NO  \n  will be append at the end of the
			       message text!
	        PNAM           (char) name of the calling program
	        PLEN           (int) length of PNAM
		CBUF           (char) message text
	        CLEN           (int) length of CBUF
 .OUTPUT	terminal and MIDAS-log output
 .RETURN	-1: the message identifier is wrong
                0:  no error occured
 .PURPOSE	Put a formatted string to stdout. The message identifier
                controls the format of the messagetext:

		id  | format                   | BELL 
		----+--------------------------+-----
		1   | ERROR   [prgname]: text  | ON
		2   | WARNING [prgname]: text  | OFF
		3   | INFO    [prgname]: text  | OFF
		4   | INFO    [prgname]: text  | OFF
 
		In case the identifier is the negative compliment then
		NO  \n  will be appended to the text.
		Following printf() may continue at that point.
 .ALGORITHM	The string passed through messagetext will be formatted and
                send to stdout (printf()). Furthermore this messagetext will
		be logged in the standard MIDAS log.
		The format of the messagetext depends on the message
		identifier id.
 .ENVIRON	MIDAS
 .LANGUAGE	C
 .AUTHOR	Sebastian Wolf, ESO-DMO
 .VERSION	1.0 1998.01.05 created.
		moved into the Midas library (with bleeding heart...) KB

 021029		last modif	KB

 .COMMENT       The message may contain special characters to format your
                text:   \n, \r, \t, \b
-----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>


#define	BELL  0x7
#define MXID  5
#define MXLN  12
#define MAX_TBUF  100
#define MAXPLEN  16
#define MAXP MAXPLEN + 3
#define OFFSET   MAXPLEN + MXLN


int  indent = 0; 

char  Mtyp[MXID][MXLN] = {"","ERROR   ","WARNING ","INFO    ","INFO    "};


/*

*/

/* +++++++++++++++++++++++++++++++++++++++++++++++++

 wrapper for message() to handle the "extern int indent"

   +++++++++++++++++++++++++++++++++++++++++++++++++ */

int MID_message(id,nam,buf,indnt)
int  id, indnt;
char *nam, *buf;

{
indent = indnt;				/* needed to get same behaviour
					   as before with extenal `indent' */
return (message(id,nam,buf));
}


/* ----------------------------------- */
/* interface for old pipeline programs */
/* ----------------------------------- */

int messout(messlevel,id,nam,buf)
int  id, messlevel;
char *nam, *buf;

{
id = abs(id);
if (id > messlevel)
   return(-1);
else
   return (message(id,nam,buf));
}
/*

*/

/*+++ message +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .IDENT		message()
 .KEYWORDS	Put a formatted string to stdout and MIDAS-log
 .USAGE         message(messlevel, id, prgname, messagetext)
 .INPUT		id             (int) message identifier
		               ( 1: ERROR, 2: WARNING, 3,4: INFO)
			       negative id has the same meaning except that
			       NO  \n  will be append at the end of the
			       message text!
	        *prgname       (char) name of the calling program
		*messagetext   (char) message text
 .OUTPUT	terminal and MIDAS-log output
 .RETURN	-1: the message identifier is wrong
                0:  no error occured
 .PURPOSE	Put a formatted string to stdout. The message identifier
                controls the format of the messagetext:

		id  | format                   | BELL 
		----+--------------------------+-----
		1   | ERROR   [prgname]: text  | ON
		2   | WARNING [prgname]: text  | OFF
		3   | INFO    [prgname]: text  | OFF
		4   | INFO    [prgname]: text  | OFF
 
 .ALGORITHM	The string passed through messagetext will be formatted and
                send to stdout (printf()). Furthermore this messagetext will
		be logged in the standard MIDAS log.
		The format of the messagetext depends on the message
		identifier id.
 .COMMENT       The message may contain special characters to format your
                text:   \n, \r, \t, \b
-----------------------------------------------------------------------------*/


int message(id, pnam, line)
int  id;
char *pnam, *line;

{
char bell, *myptr;
char  txt[124], mes[68];

int   n, mm, plen;





if (n > 64)
   {
   SCTPUT("Warning: message > 64 chars - will be truncated ...");
   mes[64] = '\0';
   }

if (id == 1)			/* for errors we beep! */
   bell = BELL;
else
   bell = '\0';

plen = (int)strlen(pnam);
if (plen>MAXPLEN)
   {
   pnam[MAXPLEN] = '\0';	/* cut program name if too long */
   plen = MAXPLEN;
   }

myptr = line;			/* point to begin of message buffer */
mm = check_special(myptr,mes);

n = sprintf(txt,"%-8s[%-19s%s%c",&Mtyp[id],pnam,mes,bell);
txt[plen+9] = ']';
txt[plen+10] = ':';
(void) printf("%s",txt);
if (mm > 0) 
   txt[--n] = '\0';		/* remove special chars */
MID_LOG('G',txt,(int)strlen(txt));
if (mm == 0)
   {
   printf("\n");
   return (0);
   }


while (mm > 0)
   {
   myptr += (mm+1);
   if (*myptr == '\0') break;

   mm = check_special(myptr,mes);
   CGN_FILL(txt,' ',OFFSET+indent);
   (void) strcpy(&txt[OFFSET+indent],mes);
   (void) printf("%s",txt);
   n = (int) strlen(txt);
   if (mm > 0) txt[--n] = '\0';		/* remove special chars */
   MID_LOG('G',txt,n);
   if (mm == 0) printf("\n");
   }

return(0);
}

int check_special(cbuf,ccbuf)
char *cbuf, *ccbuf;

{
int i, j, n;




n = (int) strlen(cbuf);
for (j=0; j<n; j++)
   {
   if (cbuf[j] == '\\')
      {
      if (cbuf[j+1] == 'n')
         {
         ccbuf[j++] = '\n';
         ccbuf[j] = '\0';
         return (j);
         }
      else if (cbuf[j+1] == 'r')
         {
         ccbuf[j++] = '\r';
         ccbuf[j] = '\0';
         return (j);
         }
      else if (cbuf[j+1] == 't')
         {
         ccbuf[j++] = '\t';
         ccbuf[j] = '\0';
         return (j);
         }
      }

   ccbuf[j] = cbuf[j];
   }
ccbuf[n] = '\0';
return (0);
}

